/****************************************************
 Statistics Online Computational Resource (SOCR)
 http://www.StatisticsResource.org
  
 All SOCR programs, materials, tools and resources are developed by and freely disseminated to the entire community.
 Users may revise, extend, redistribute, modify under the terms of the Lesser GNU General Public License
 as published by the Open Source Initiative http://opensource.org/licenses/. All efforts should be made to develop and distribute
 factually correct, useful, portable and extensible resource all available in all digital formats for free over the Internet.
  
 SOCR resources are distributed in the hope that they will be useful, but without
 any warranty; without any explicit, implicit or implied warranty for merchantability or
 fitness for a particular purpose. See the GNU Lesser General Public License for
 more details see http://opensource.org/licenses/lgpl-license.php.
  
 http://www.SOCR.ucla.edu
 http://wiki.stat.ucla.edu/socr
  It s Online, Therefore, It Exists! 
 ****************************************************/

package edu.ucla.stat.SOCR.TG_distributome.editor;

import javax.swing.table.AbstractTableModel;

/**
 * Abstract Table Model for displaying the attributes of an provenance element.
 *
 * @version 10 September 2007
 */
public class ElementTableModel extends AbstractTableModel
{
    /** Name of the element. */
    private String _elementName;

    /** Names of the attributes. */
    private String[] _attrNames;

    /** Values of the attributes. */
    private String[] _attrValues;

    /**
     * Constructs an Element Table Model.
     *
     * @param elementName Name of the element.
     * @param attrNames Names of the attributes of the element.
     */
    public ElementTableModel(String elementName, String[] attrNames)
    {
	_elementName = elementName;
	_attrNames = attrNames;

	// Initialize empty values
	_attrValues = new String[_attrNames.length];
	for (int i = 0; i < _attrValues.length; i++) { _attrValues[i] = ""; }
    }

    /**
     * Gets the name of the element.
     *
     * @return Name of the element.
     */
    public String getElementName()
    {
	return _elementName;
    }

    /**
     * Gets the names of the attributes.
     *
     * @return Names of the attributes.
     */
    public String[] getAttributeNames()
    {
	return _attrNames;
    }

    /**
     * Gets the value of the attribute.
     *
     * @param attrName Name of the attribute.
     *
     * @return Value of the attribute.
     */
    public String getAttributeValue(String attrName)
    {
	for (int i = 0; i < _attrNames.length; i++) {
	    if ( _attrNames[i].equals(attrName) ) { return _attrValues[i]; }
	}

	// Attribute not found
	return "";
    }

    /**
     * Sets the value of the attribute.
     *
     * @param attrName Name of the attribute.
     * @param attrValue Value for the attribute.
     */
    public void setAttribute(String attrName, String attrValue)
    {
	// Set the attribute
	for (int i = 0; i < _attrNames.length; i++) {
	    if ( _attrNames[i].equals(attrName) ) {
		setValueAt(attrValue, i+1, 1);
	    }
	}
    }

    /**
     * Returns the number of rows in the model.
     *
     * @return Number of rows in the model.
     */
    public int getRowCount()
    {
	return _attrNames.length + 1;
    }

    /**
     * Returns the number of columns in the model.
     *
     * @return Number of columns in the model.
     */
    public int getColumnCount()
    {
	return 2;
    }

    /**
     * Returns the name of the specified column.
     *
     * @param columnIndex Index of the column.
     *
     * @return Name of the column.
     */
    public String getColumnName(int columnIndex)
    {
	if (columnIndex == 0) { return "Name"; }
	return "Value";
    }

    /**
     * Returns the most specific superclass for all the cell values 
     * in the column.
     *
     * @param columnIndex Index of the column.
     *
     * @return Common ancestor class of the object values in the model.
     */
    public Class getColumnClass(int columnIndex)
    {
	return String.class;
    }

    /**
     * Determines if the specified table cell is editable.
     *
     * @param rowIndex Index of the row.
     * @param columnIndex Index of the column.
     *
     * @return True if the table cell is editable; false otherwise.
     */
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
	if (columnIndex == 0) { return false; }
	return true;
    }

    /**
     * Returns the value of the specified table cell.
     *
     * @param rowIndex Index of the row.
     * @param columnIndex Index of the column.
     *
     * @return Value of the specified table cell.
     */
    public Object getValueAt(int rowIndex, int columnIndex)
    {
	// Name
	if (columnIndex == 0) {
	    if (rowIndex == 0) {
		return "<html><b>" + _elementName + "</b></html>";
	    }
	    return _attrNames[rowIndex-1];
	}

	// Value
	if (rowIndex == 0) { return ""; }
	return _attrValues[rowIndex-1];
    }
	
    /**
     * Sets the value of the specified table cell.
     *
     * @param newValue New value for the table cell.
     * @param rowIndex Index of the row.
     * @param columnIndex Index of the column.
     */
    public void setValueAt(Object newValue, int rowIndex, int columnIndex)
    {
	if (columnIndex == 0) { return; }
	if (rowIndex == 0) { return; }

	_attrValues[rowIndex-1] = newValue.toString();
    }
}
